<?php
namespace Drupal\entity_expand;


use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityListBuilder;
use Drupal\Core\Field\FieldItemListInterface;

class EntityExpandBase implements EntityExpandInterface {

  /**
   * @var EntityInterface
   */
  protected EntityInterface $entity;

  /**
   * init entity.
   *
   * Entity constructor.
   * @param EntityInterface $entity
   */
  public function __construct(EntityInterface $entity) {
    $this->entity = $entity;
    return $this->entity;
  }

  public function getEntity() {
    return $this->entity;
  }

  /**
   * get field use custom.
   *
   * @param $field
   * @return FieldItem
   */
  public function get($field):FieldItem {
    return new FieldItem($this->entity, $field);
  }

  /**
   * bundle key
   * node key is type
   * profile key is profile_type
   * @return false|string
   */
  public function bundleKey() {
    return $this->entity->getEntityType()->getKey('bundle');
  }

  /**
   * $values = [
   *   'field1' => 'value1',
   *   'field2' => 'value2',
   * ]
   * @param $values
   */
  public function setValues($values) {
    foreach ($values as $field => $value) {
      if ($this->entity->hasField($field)) {
        $this->entity->set($field, $value);
      }
    }
  }

  /**
   * build entity view element.
   *
   * @param string $view_mode
   * @return array
   */
  public function view($view_mode = 'full') {
    return \Drupal::entityTypeManager()
      ->getViewBuilder($this->entity->getEntityTypeId())
      ->view($this->entity, $view_mode);
  }

  /**
   * get entity operations.
   * [
   *   'view' => [],
   *   'edit' => [],
   *   'delete' => []
   * ]
   *
   * @return mixed
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function getOperations() {
    $entityTypeManager = \Drupal::entityTypeManager();
    $entity_type = $entityTypeManager->getDefinition($this->entity->getEntityTypeID());
    $list_builder = $entityTypeManager->createHandlerInstance(EntityListBuilder::class, $entity_type);
    return $list_builder->getOperations($this->entity);
  }

  /**
   * call entity old functions.
   *
   * @param $name
   * @param $arguments
   * @return false|mixed
   */
  public function __call($name, $arguments)
  {
    if (method_exists($this->entity, $name)) {
      return call_user_func_array(array(
        $this->entity,
        $name
      ), $arguments);
    }
    return false;
  }

  /**
   * call entity old property.
   *
   * @param $name
   * @return FieldItem
   */
  public function __get($name) {
    return new FieldItem($this->entity, $name);
  }
}

/**
 * entity field item.
 * Class FieldItem
 * @package Drupal\dyniva_helper
 */
class FieldItem {

  /**
   * @var FieldItemListInterface
   */
  protected FieldItemListInterface $field;

  /**
   * FieldItem constructor.
   * @param EntityInterface $entity
   * @param $field
   */
  public function __construct(EntityInterface $entity, $field) {
    $this->field = $entity->get($field);
  }

  /**
   * set call field old functions.
   *
   * @param $name
   * @param $arguments
   * @return false|mixed
   */
  public function __call($name, $arguments) {
    if (method_exists($this->field, $name)) {
      return call_user_func_array(array(
        $this->field,
        $name
      ), $arguments);
    }
    return false;
  }

  /**
   * set call field old property.
   *
   * @param $name
   * @return mixed
   */
  public function __get($name) {
    if ($name == 'entity') {
      return $this->ref();
    }
    return $this->field->{$name};
  }

  public function emptyValue($default = '') {
    if (!$this->field->isEmpty()) {
      return $this->field->getString();
    }
    return $default;
  }

  /**
   * get list[Text] field label, drupal default privide key.
   * @return string
   */
  public function listTextLabel($default_value = '') {
    $allowvalues = $this->field->getSetting('allowed_values');
    return $allowvalues[$this->field->value] ?? $default_value;
  }

  /**
   * get ref entity.
   *
   * @param null $default_value
   * @return EntityInterface|null
   */
  public function ref($default_value = NULL) {
    if (!$this->field->isEmpty() && $this->field->entity) {
      return entity_expand_load($this->field->entity);
    }
    return $default_value;
  }

  /**
   * @param string $default_value
   * @return FieldItemListInterface|mixed|string
   */
  public function val($default_value = '') {
    return !$this->field->isEmpty() ? $this->field : $default_value;
  }

  /**
   * get field values.
   *
   * @return array
   */
  public function vals($default_value = []) {
    if ($this->field->isEmpty()) {
      return [];
    }
    return array_column($this->field->getValue(), 'value');
  }

  /**
   * get refs entity.
   *
   * @param callable|null $callable
   * @return array
   */
  public function refs(callable $callable = NULL) {
    if (!$this->field->isEmpty()) {
      $entitys = array_map(function($entity) {
        return entity_expand_load($entity);
      }, $this->field->referencedEntities());

      if ($callable) {
        array_map($callable, $entitys);
      }
      return $entitys;
    }

    return [];
  }

  /**
   * get field targets.
   *
   * @return array
   */
  public function targets() {
    if ($this->field->isEmpty()) {
      return [];
    }
    return array_column($this->field->getValue(), 'target_id');
  }
}
